@@ -62,6 +62,8 @@ class auto_timeseries:
         verbose: int = 0,
         dask_xgboost_flag: int = 0,
         lag: int = 3,
+        strf_time_format = '',
+        num_boost_rounds = 250,
         *args,
         **kwargs
     ):
@@ -151,13 +153,12 @@ class auto_timeseries:
         :param lag Indicates the number of lags to include in ML models (Default = 2)
         :type verbose int
 
-
-        ##################################################################################################
-        AUTO_TIMESERIES IS A VERY COMPLEX MODEL BUILDING UTILITY FOR TIME SERIES DATA. SINCE IT AUTOMATES 
-        MANY TASKS INVOLVED AT A FAST PACE, IT ASSUMES INTELLIGENT DEFAULTS. BUT YOU CAN CHANGE THEM.
-        Auto_Timeseries will rapidly build predictive models based on Statsmodels, auto-ARIMA, Scikit-Learn
-        and now dask_xgboost. It will automatically select the BEST model which gives best score specified.
-        #####################################################################################################
+        #   ##################################################################################################
+        #    AUTO_TIMESERIES IS A VERY COMPLEX MODEL BUILDING UTILITY FOR TIME SERIES DATA. SINCE IT AUTOMATES 
+        #    MANY TASKS INVOLVED AT A FAST PACE, IT ASSUMES INTELLIGENT DEFAULTS. BUT YOU CAN CHANGE THEM.
+        #    Auto_Timeseries will rapidly build predictive models based on Statsmodels, auto-ARIMA, Scikit-Learn
+        #    and now dask_xgboost. It will automatically select the BEST model which gives best score specified.
+        #    #####################################################################################################            
         """
         self.ml_dict: Dict = {}
         self.score_type: str = score_type
@@ -176,14 +177,19 @@ class auto_timeseries:
         self.allowed_models = ['best', 'prophet', 'stats', 'ml', 'arima','ARIMA','Prophet','SARIMAX', 'VAR', 'ML']
         self.dask_xgboost_flag = dask_xgboost_flag
         self.sep = ','
-        self.strf_time_format = ''
+        self.strf_time_format = strf_time_format
+        self.num_boost_rounds = num_boost_rounds
         self.lag = lag
+        if self.dask_xgboost_flag:
+            print("### Be careful setting 'dask_xgboost_flag' to True since dask is unstable and doesn't work well ###")
 
         # new function.
         if args:
             for each_arg in args:
                 print(each_arg)
+
         if kwargs:
+            self.kwargs = kwargs
             for key, value in zip(kwargs.keys(), kwargs.values()):
                 if key == 'seasonal_PDQ':
                     print('seasonal_PDQ argument is deprecated. Please remove the argument in future.')
@@ -193,6 +199,8 @@ class auto_timeseries:
                 if key == 'growth':
                     print('growth argument of FB Prophet given. It must be "linear" or "logistic"')
                     self.growth = value
+        else:
+            self.kwargs = {}
 
     def fit(
         self,
@@ -366,10 +374,13 @@ class auto_timeseries:
                 time_series_plot(ts_df[target], lags=31, title='Original Time Series after %s differencing' %diff_limit,
                     chart_type='line', chart_freq=self.time_interval)
             else:
-                time_series_plot(ts_df[target], lags=31, title='Original Time Series',
+                if ts_df.shape[0] <= 100000:
+                    time_series_plot(ts_df[target], lags=31, title='Original Time Series',
                         chart_type='line', chart_freq=self.time_interval)
+                else:
+                    print('    No time series plot since number of rows > 100K. Continuing...')
         else:
-            print('No time series plot since verbose = 0. Continuing')
+            print('    No time series plot since verbose = 0. Continuing')
         ##################################################################################################
         ### Turn the time series index into a variable and calculate the difference.
         ### If the difference is not in days, then it is a hourly or minute based time series
@@ -541,7 +552,7 @@ class auto_timeseries:
                 model_build = BuildProphet(
                     self.forecast_period, self.time_interval, self.seasonal_period,
                     self.score_type, self.verbose, self.conf_int, self.holidays, self.growth,
-                    self.seasonality)
+                    self.seasonality, **self.kwargs)
                 model, forecast_df_folds, rmse_folds, norm_rmse_folds = model_build.fit(
                     ts_df=ts_df[[target]+preds],
                     target_col=target,
@@ -559,6 +570,7 @@ class auto_timeseries:
                 print("Exception occurred while building Prophet model...")
                 print(e)
                 print('    FB Prophet may not be installed or Model is not running...')
+                forecast_df_folds = None
 
             self.ml_dict[name]['model'] = model
             self.ml_dict[name]['forecast'] = forecast_df_folds
@@ -657,7 +669,7 @@ class auto_timeseries:
             self.ml_dict[name]['model_build'] = model_build
 
 
-        if self.__any_contained_in_list(what_list=['var','Var','VAR', 'stats', 'best'], in_list=self.model_type):
+        if self.__any_contained_in_list(what_list=['var','Var','VAR', 'best'], in_list=self.model_type):
             ########### Let's build a VAR Model - but first we have to shift the predictor vars ####
 
             if ts_df.shape[0] > 1000 and self.__any_contained_in_list(what_list=['stats', 'best'], in_list=self.model_type):
@@ -755,6 +767,8 @@ class auto_timeseries:
                 time_interval = self.time_interval,
                 sep = self.sep,
                 dask_xgboost_flag = self.dask_xgboost_flag,
+                strf_time_format = self.strf_time_format,
+                num_boost_rounds = self.num_boost_rounds,
                 verbose=self.verbose)
             try:
                 if self.dask_xgboost_flag:
@@ -959,12 +973,12 @@ class auto_timeseries:
             if isinstance(testdata, pd.Series) or isinstance(testdata, pd.DataFrame):
                 # During training, we internally converted a column datetime index to the dataframe date time index
                 # We need to do the same while predicing for consistence
-                if (model == 'ML') or self.get_best_model_name() == 'ML' or (model == 'best' and self.get_best_model_name() == 'ML'):
+                if (model.lower() == 'ml') or self.get_best_model_name().lower() == 'ml' or (model.lower() == 'best' and self.get_best_model_name().lower() == 'ml'):
                     ### Now do the predictions using the final model asked to be predicted ###
                     predictions = bestmodel.predict(testdata,simple=simple, time_interval=time_interval)
                 elif model.lower() == 'prophet' or self.get_best_model_name() == 'Prophet':
                     predictions = bestmodel.predict(testdata,simple=simple)
-                elif self.get_best_model_name() == 'VAR':
+                elif self.get_best_model_name().lower() == 'var':
                     predictions = bestmodel.predict(testdata,simple=simple)
                 else:
                     predictions = bestmodel.predict(testdata,simple=simple)
