@@ -16,10 +16,10 @@ import dask.dataframe as dd
 
 import matplotlib.pyplot as plt # type: ignore
 
-from fbprophet import Prophet # type: ignore
-from fbprophet.diagnostics import cross_validation
-from fbprophet.diagnostics import performance_metrics
-from fbprophet.plot import plot_cross_validation_metric
+from prophet import Prophet # type: ignore
+from prophet.diagnostics import cross_validation
+from prophet.diagnostics import performance_metrics
+from prophet.plot import plot_cross_validation_metric
 
 from .build_base import BuildBase
 
@@ -29,7 +29,7 @@ from ..utils.logging import SuppressStdoutStderr
 
 #### Suppress INFO messages from FB Prophet!
 
-logging.getLogger('fbprophet').setLevel(logging.WARNING)
+logging.getLogger('prophet').setLevel(logging.WARNING)
 
 
 class BuildProphet(BuildBase):
@@ -45,7 +45,8 @@ class BuildProphet(BuildBase):
         super().__init__(
             scoring=scoring,
             forecast_period=forecast_period,
-            verbose=verbose
+            verbose=verbose,
+            kwargs = kwargs
         )
         self.time_interval = time_interval
         self.seasonal_period = seasonal_period
@@ -76,15 +77,19 @@ class BuildProphet(BuildBase):
                                         'A,Y','BA,BY','AS,YS','BAS,BYS','BH',
                                         'H','T,min','S','L,ms','U,us','N']
         self.list_of_valid_time_ints.append(time_interval)
-
+        
         if kwargs:
+            self.kwargs = kwargs
             for key, value in zip(kwargs.keys(),kwargs.values()):
                 if key == 'seasonality_mode':
                     self.seasonality = True
                     key = value
                 else:
                     key = value
-
+        else:
+            self.kwargs = {'iter':100}
+        print('kwargs for Prophet model: %s' %self.kwargs)
+        
     def fit(self, ts_df: pd.DataFrame, target_col: str, cv: Optional[int], time_col: str):
         """
         Fits the model to the data
@@ -231,7 +236,8 @@ class BuildProphet(BuildBase):
         if  cv_in == 0:
             print('Skipping cross validation steps since cross_validation = %s' %cv_in)
             model = Prophet(growth="linear")
-            kwargs = {'iter':1e2} ## this limits iterations and hence speeds up prophet
+
+            kwargs = self.kwargs ## this limits iterations and hence speeds up prophet
         else:
             for fold_number, (train_index, test_index) in enumerate(cv.split(dft)):
                 dftx = dft.head(len(train_index)+len(test_index))
@@ -251,7 +257,7 @@ class BuildProphet(BuildBase):
                 #### Fit the model with train_fold data ####
                 ############################################
 
-                kwargs = {'iter':1e2} ## this limits iterations and hence speeds up prophet
+                kwargs = {'iter':100} ## this limits iterations and hence speeds up prophet
                 model.fit(train_fold, **kwargs)
 
                 #################################################
@@ -293,7 +299,7 @@ class BuildProphet(BuildBase):
             try:
                 print_ts_model_stats(extra_concatenated['original'], extra_concatenated['predicted'], "Prophet")
             except:
-                print('Error: Not able to plot Prophet CV results')
+                print('Not able to plot Prophet CV results. Continuing...')
 
             forecast_df_folds = extra_concatenated['predicted'].values
             #print("  End of Prophet Cross Validation")
@@ -536,7 +542,7 @@ def plot_prophet(dft, forecastdf):
     return viz_df
 #################################
 from sklearn.metrics import mean_squared_error
-from fbprophet import Prophet
+from prophet import Prophet
 import time
 import pdb
 import copy
@@ -563,7 +569,7 @@ def easy_cross_validation(train, target, initial, horizon, period):
         if len(test_fold) == 0:
             break
         model = Prophet(growth="linear")
-        kwargs = {'iter':1e2} ## this limits iterations and hence speeds up prophet
+        kwargs = {'iter':100} ## this limits iterations and hence speeds up prophet
         model.fit(train_fold, **kwargs)
         future_period = model.make_future_dataframe(freq="MS",periods=horizon)
         forecast_df = model.predict(future_period)
