@@ -15,7 +15,7 @@ import pdb
 import copy
 import dask
 import dask.dataframe as dd
-
+from .colors import colorful
 def time_series_plot(y, lags=31, title='Original Time Series', chart_type='line',
                      chart_freq='years'):
     """
@@ -103,6 +103,8 @@ def time_series_plot(y, lags=31, title='Original Time Series', chart_type='line'
     y.plot(ax=hist_ax, kind='hist', bins=25, color=next(colors))
     hist_ax.set_title('Histogram for Original Series')
     try:
+        if len(y) < lags:
+            lags = int(len(y) - 1)
         smt.graphics.plot_acf(y, lags=lags, ax=acf_ax)
         acf_ax.set_title('ACF for Original Series')
     except:
@@ -215,12 +217,15 @@ def test_stationarity(time_df, maxlag=31, regression='c', autolag='BIC',
     window argument is only required for plotting rolling functions. Default=4.
     """
     time_df = copy.deepcopy(time_df)
-    if len(time_df) <= int(1.5*maxlag):
-        maxlag = 5  ## set it to a low number
+    if len(time_df) <= int(2.5*maxlag):
+        maxlag = 5 
+        print('setting maxlag to a low number = %s' %maxlag) 
     # set defaults (from function page)
     if type(time_df) == pd.DataFrame:
         #print('modifying time series dataframe into an array to test')
         timeseries = time_df.values.ravel()
+    else:
+        timeseries = copy.deepcopy(time_df)
     if regression is None:
         regression = 'c'
     if verbose:
@@ -276,7 +281,6 @@ def test_stationarity(time_df, maxlag=31, regression='c', autolag='BIC',
         return diff_limit
     else:
         ### In non-VAR models you need to test only the target variable for stationarity ##
-        timeseries = copy.deepcopy(time_df)
         dftest = smt.adfuller(timeseries, maxlag=maxlag, regression=regression, autolag=autolag)
         dfoutput = pd.Series(dftest[0:4], index=['Test Statistic',
                                                  'p-value',
@@ -303,13 +307,13 @@ def test_stationarity(time_df, maxlag=31, regression='c', autolag='BIC',
                 print('After differencing=1, results of Augmented Dickey-Fuller Test:')
                 pretty_print_table(dfoutput)
             if dftest[1] >= alpha:
-                print(' this series is not stationary')
+                print(colorful.BOLD +'this series is NOT stationary' + colorful.END)
                 return False
             else:
-                print(' this series is stationary')
+                print(colorful.BOLD +'this series is stationary' + colorful.END)
                 return True
         else:
-            print(' this series is stationary')
+            print(colorful.BOLD +'this series is stationary' + colorful.END)
             return True
 ################################################################################
 def adjust(val, length= 6): 
